#!/usr/bin/python
#-*- coding: utf-8 -*-
"""
rst2textile.

:copyright: Copyright 2012 by Takayuki SHIMIZUKAWA.
:license: Apache License 2.0.
"""

__docformat__ = 'reStructuredText'

if __name__ == '__main__':
    import sys
    from docutils_textile.main import main
    sys.exit(main())



# -*- coding: utf8 -*-
"""
docutils textile builder.

:copyright: Copyright 2012 by Takayuki SHIMIZUKAWA.
:license: Apache Software License
"""

__docformat__ = 'reStructuredText'

import os

from docutils import nodes
try:
    from sphinx.builders.text import TextBuilder
    from sphinx.util.osutil import relative_uri, ensuredir, copyfile
    from sphinx.util.console import brown
except ImportError:
    raise ImportError("sphinx: you NEED Sphinx installation if you want to use `docutils_textile.builder`.")

from docutils_textile.writer import TextileWriter, TextileTranslator


class SphinxTextileTranslator(TextileTranslator):

    def __init__(self, document, builder):
        TextileTranslator.__init__(self, document)
        self.builder = builder

    def unknown_visit(self, node):
        msg = ('%s visiting unknown node type: %s'
               % (self.__class__, node.__class__.__name__))
        self.builder.warn(msg, (self.builder.current_docname, node.line))
        TextileTranslator.unknown_visit(self, node)


class SphinxTextileWriter(TextileWriter):
    def __init__(self, builder):
        TextileWriter.__init__(self)
        self.builder = builder

    def translate(self):
        visitor = SphinxTextileTranslator(self.document, self.builder)
        self.document.walkabout(visitor)
        self.output = visitor.body


class SphinxTextileBuilder(TextBuilder):
    name = 'textile'
    format = 'text'
    out_suffix = '.txt'

    def prepare_writing(self, docnames):
        self.writer = SphinxTextileWriter(self)

    def write_doc(self, docname, doctree):
        self.imgpath = relative_uri(self.get_target_uri(docname), '_images')
        self.post_process_images(doctree)
        self.current_docname = docname
        return TextBuilder.write_doc(self, docname, doctree)

    def post_process_images(self, doctree):
        for node in doctree.traverse(nodes.image):
            candidate = node['uri']
            if candidate.startswith(self.outdir):
                candidate = candidate[len(self.outdir):]
            candidate = candidate.replace('\\', '/').lstrip('/')
            node['uri'] = candidate
            if candidate not in self.env.images:
                # non-existing URI; let it alone
                continue
            self.images[candidate] = self.env.images[candidate][1]

    def finish(self):
        self.copy_image_files()
        return TextBuilder.finish(self)

    def copy_image_files(self):
        # copy image files
        if self.images:
            ensuredir(os.path.join(self.outdir, '_images'))
            for src in self.status_iterator(self.images, 'copying images... ',
                                            brown, len(self.images)):
                dest = self.images[src]
                try:
                    copyfile(os.path.join(self.srcdir, src),
                             os.path.join(self.outdir, '_images', dest))
                except Exception, err:
                    self.warn('cannot copy image file %r: %s' %
                              (os.path.join(self.srcdir, src), err))



"""
docutils textile.

:copyright: Copyright 2012 by Takayuki SHIMIZUKAWA.
:license: Apache Software License
"""

__docformat__ = 'reStructuredText'

from docutils_textile.writer import TextileWriter as Writer  #for docutils rule.
    #if docutils receive 'foo' as writer name, execute `import foo.Writer`.


# setup hook for sphinx
def setup(app):
    from docutils_textile.builder import SphinxTextileBuilder
    app.add_builder(SphinxTextileBuilder)  #sphinx builder registration



#!/usr/bin/python
#-*- coding: utf-8 -*-
"""
docutils textile.

:copyright: Copyright 2012 by Takayuki SHIMIZUKAWA.
:license: Apache Software License
"""

__docformat__ = 'reStructuredText'

import docutils.core

description = """`rst2textile` is docutils textile writer convert reStructuredText(rst) to Textile format."""


def main():
    output = docutils.core.publish_cmdline(
            writer_name='docutils_textile',
            usage='usage',
            description=description,
            )
    return 0

if __name__ == '__main__':
    main()



import sys
import unittest
import textwrap
import docutils.core
import docutils.nodes
import docutils.writers


if sys.version_info >= (3, 0):
    py3 = True
else:
    py3 = False


WRITER_ALIASES = {
    'textile': 'docutils_textile',
}


class ReSTTestCase(unittest.TestCase):
    def __init__(self, _from, _to, source, expect, actual, node_tree):
        unittest.TestCase.__init__(self)
        self.source = source
        self.expect = expect
        self.actual = actual
        self.node_tree = node_tree
        self._from = _from
        self._to = _to

    def runTest(self):
        message = textwrap.dedent("""\
        Convertion Mismatch (%s -> %s)
        ##source:
        %s

        ##expect:
        %s

        ##actual:
        %s

        ##node-tree:
        %s
        """)
        message = message % (self._from, self._to, self.source, self.expect, self.actual, self.node_tree)
        self.assertEqual(self.expect, self.actual, message)


class DocutilsTestTranslator(docutils.nodes.NodeVisitor):

    def __init__(self, document):
        docutils.nodes.NodeVisitor.__init__(self, document)
        self.body = []

    def unknown_visit(self, node):
        pass

    def unknown_departure(self, node):
        pass

    def visit_container(self, node):
        if 'classes' in node and 'test' in node['classes']:
            literals = [x for x in node.children if x.tagname=='literal_block']
            if len(literals) < 2:
                raise RuntimeError('Need 2 literal-block elements "from", "to"')

            _, _from, _to = node['classes'][:3]
            _to = WRITER_ALIASES.get(_to, _to)
            source_node, expect_node = literals[:2]  #discard over 2 nodes.
            source = source_node.astext()
            expect = expect_node.astext()
            actual = docutils.core.publish_string(
                    source,
                    parser_name=_from,
                    writer_name=_to,
                    settings_overrides={'output_encoding': 'unicode'})
            actual = actual.strip()
            node_tree = docutils.core.publish_string(
                    source,
                    parser_name=_from,
                    writer_name="pseudoxml").strip()
            self.body.append(ReSTTestCase(
                _from, _to, source, expect, actual, node_tree))

        raise docutils.nodes.SkipNode


class DocutilsTestWriter(docutils.writers.Writer):
    def translate(self):
        visitor = DocutilsTestTranslator(self.document)
        self.document.walkabout(visitor)
        self.output = visitor.body


class ReSTTestSuite(unittest.TestSuite):

    def __init__(self, filename):
        f = open(filename, 'rt')
        data = f.read()
        if not py3:
            data = data.decode('utf8')
        tests = docutils.core.publish_string(
                source=data, source_path=f.name,
                writer=DocutilsTestWriter())
        f.close()
        unittest.TestSuite.__init__(self, tests)


def suite():
    suite = ReSTTestSuite('README.rst')
    return suite


if __name__ == '__main__':
    unittest.main(defaultTest='suite')
